@@ -309,7 +309,7 @@ class SafeRepresenter(BaseRepresenter):
             value = u'-.inf'
         else:
             value = to_unicode(repr(data)).lower()
-            if self.dumper.version == (1, 1):  # type: ignore
+            if self.serializer.use_version == (1, 1):
                 if u'.' not in value and u'e' in value:
                     # Note that in some cases `repr(data)` represents a float number
                     # without the decimal parts.  For instance:
@@ -652,7 +652,7 @@ class RoundTripRepresenter(SafeRepresenter):
 
     def __init__(self, default_style=None, default_flow_style=None, dumper=None):
         # type: (Any, Any, Any) -> None
-        if default_flow_style is None:
+        if not hasattr(dumper, 'typ') and default_flow_style is None:
             default_flow_style = False
         SafeRepresenter.__init__(self, default_style=default_style,
                                  default_flow_style=default_flow_style,
@@ -844,6 +844,16 @@ class RoundTripRepresenter(SafeRepresenter):
         best_style = True
         try:
             comment = getattr(sequence, comment_attrib)
+            node.comment = comment.comment
+            # reset any comment already printed information
+            if node.comment and node.comment[1]:
+                for ct in node.comment[1]:
+                    ct.reset()
+            item_comments = comment.items
+            for v in item_comments.values():
+                if v and v[1]:
+                    for ct in v[1]:
+                        ct.reset()
             item_comments = comment.items
             node.comment = comment.comment
             try:
@@ -982,6 +992,7 @@ class RoundTripRepresenter(SafeRepresenter):
         for item_key in omap:
             item_val = omap[item_key]
             node_item = self.represent_data({item_key: item_val})
+            # node_item.flow_style = False
             # node item has two scalars in value: node_key and node_value
             item_comment = item_comments.get(item_key)
             if item_comment:
